import curses

from BilaPani import BilaPani
from Cestar import Cestar
from Televize import Televize
from Zved import Zved

class Bludiste:
    ##############################################
    # Inicializace                               #
    ##############################################
    def __init__( self, soubor ):
        # Krok
        self.step     = 0
        # Seznam zvedu
        self.zvedi    = []
        # Bila pani
        self.bilaPani = None
        # Televize
        self.televize = None
        # Hraci pole
        self.pole     = []

        # Nacti prvni radek
        rozmery = soubor.readline().split()

        # Pamatuj rozmery
        sloupcu = int(rozmery[0])
        radku   = int(rozmery[1])

        # Iteruj pres radky
        for y in range( radku ):
            # Pridek radek
            self.pole.append( [] )

            # Iteruj pres sloupce
            for x in range(sloupcu):
                c = soubor.read(1)

                # Pokud je to entita, vytvor objekt
                if Zved.ZNAK == c:
                    self.zvedi.append( Zved( soubor, x, y ) )
                elif BilaPani.ZNAK == c:
                    self.bilaPani = BilaPani( x, y )
                elif Televize.ZNAK == c:
                    self.televize = Televize( x, y )

                # Uloz do hraciho pole
                self.pole[y].append( c )

            # Precti konec radky
            if '\n' != soubor.read(1):
                raise ValueError( "Chybny vstupni soubor" )

    ##############################################
    # Informace                                  #
    ##############################################
    def sirka( self ):
        return len( self.pole[0] )

    def vyska( self ):
        return len( self.pole )

    def jeZed( self, x, y ):
        return self.pole[y][x] == 'X'

    def jeZved( self, x, y ):
        return self.pole[y][x] == Zved.ZNAK

    def jeBilaPani( self, x, y ):
        return self.pole[y][x] == BilaPani.ZNAK

    def setZved( self, x, y ):
        self.pole[y][x] = Zved.ZNAK

    def unsetZved( self, x, y ):
        if self.jeZved( x, y ):
            self.pole[y][x] = '.'

    def setBilaPani( self, x, y ):
        self.pole[y][x] = BilaPani.ZNAK

    def unsetBilaPani( self, x, y ):
        if self.jeBilaPani( x, y ):
            self.pole[y][x] = '.'

    ##############################################
    # run()                                      #
    ##############################################
    def nextStep( self ):
        self.step += 1

        self.bilaPani.nextStep( self )
        for zved in self.zvedi:
            zved.nextStep( self )

    def prevStep( self ):
        self.step = max( 0, self.step - 1 )

        self.bilaPani.prevStep( self )
        for zved in self.zvedi:
            zved.prevStep( self )

    def run( self, w ):
        c = Cestar( self )
        p = c.hledejCestu( 0, 0, False )

        if p:
            self.bilaPani.setPath( p )

        # p = c.hledejCestu( 0, 0, True )
        # if p:
        #     w.addstr( "Cesta (prochazeni zdmi):\n" )
        #     w.addstr( "  %s\n" % p )
        #     w.addstr( "  (%d kroku)\n" % len(p) )
        #     c.vytiskniCestu( w )
        #     return

        # w.addstr( "Cesta nenalezena\n" )

        while True:
            # Vytiskni pole
            self.vytiskniPole( w, c, p )

            ch = w.getch()
            if ch == curses.KEY_RIGHT:
                self.nextStep()
            elif ch == curses.KEY_LEFT:
                self.prevStep()
            else:
                break

    def vytiskniPole( self, w, c, p ):
        w.clear()
        w.move( 0, 0 )

        w.addstr( "Hraci pole (krok %d):\n" % self.step )
        for radek in self.pole:
            for ch in radek:
                w.addstr( ch + ' ' )
            w.addch( '\n' )
        w.addch( '\n' )

        if p:
            w.addstr( "Cesta:\n" )
            w.addstr( "  %s\n" % p )
            w.addstr( "  (%d kroku)\n" % len(p) )
            c.vytiskniCestu( w )
        else:
            w.addstr( "Cesta nenalezena\n" )
